<?php
/**
 * 乐刷通道 微信/支付宝支付基础API
 * @category   api
 * Created by PhpStorm.
 * User: roc
 * Date: 2019/5/30
 * Time: 11:56
 */

namespace kwy\lepass\lib\pay\request;


use kwy\lepass\lib\pay\LePassPayException;

class LePassApi{
    const API_URL='https://mobilepos.yeahka.com/cgi-bin/lepos_pay_gateway.cgi';

    protected static $_key = '6595928506A616350DC88C07778CB644';    //乐刷提供的正式机构秘钥

    protected $log_data = [];

    public function __construct(){

    }

    /**
     * 请求数据
     * @param $data
     * @param array $log_data
     * @return mixed
     */
    public function postCurl($data,$log_data = []){
        $url = self::API_URL;
        $ch = curl_init();
        $header = array();
        $header[] = 'Content-Type:application/x-www-form-urlencoded';
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_HEADER, 0);//设定是否输出页面内容
        //curl_setopt($ch,CURLOPT_HTTPHEADER,$header);
        curl_setopt($ch, CURLOPT_POST, 1); // post,get 过去
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); //不验证证书
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false); //不验证证书
        curl_setopt($ch,CURLOPT_RETURNTRANSFER,1 );
        curl_setopt($ch,CURLOPT_CONNECTTIMEOUT,10);

        //curl_setopt($ch, CURLOPT_PROXY, '192.168.0.12'); //代理服务器地址
        //curl_setopt($ch, CURLOPT_PROXYPORT, 8888); //代理服务器端口

        $start_time = microtime(true);  //开始请求时的时间
        $ret = curl_exec($ch);

        $time = microtime(true) - $start_time;  //请求消耗的时间

        //如果返回数据解析失败,那么记录日志
        if($ret === false){
//            $ip = C("SERVICE_IP");  //本机ip
            $ip = "unknow";
            $this->saveRequestData($data,$ret.'本机ip:'.$ip.'----'.curl_errno($ch).'---'.curl_error($ch),$time);
        }

        //记录请求日志
        if(!empty($log_data)){
            $this->saveRequestData($data,$ret,$time);
        }

        $result = $this->xmlToArray($ret);
        return $result;
    }

    /**
     * 获取请求参数
     * @param $param
     * @return string
     */
    protected function getPostData($param){
        ksort($param);
        $mParam = $param;
        $queryParam_arr = array();
        foreach($param as $k=>$v){
            $queryParam_arr[] = $k."=".$v;
        }
        $queryParam = implode("&",$queryParam_arr);
        $sign = strtoupper(md5($queryParam.'&key='.self::$_key));
        $mQueryParamArr = array();
        foreach($mParam as $k=>$v){
            $mQueryParamArr[$k] = $v;
        }
        $mQueryParamArr['sign'] = $sign;
        return  http_build_query($mQueryParamArr);
    }

    /**
     * 签名校验
     * @param $param
     * @return bool
     */
    public function checkSign($param){
        $own_sign = $param['sign'];
        unset($param['error_code'],$param['sign']);
        ksort($param);
        $key = '448F2A19C6463945AB1EBB38ECC879B3';
        $queryParam_arr = array();
        foreach($param as $k=>$v){
            if(empty($v) && $v != '0') $v='';
            $queryParam_arr[] = $k."=".$v;
        }
        $queryParam = implode("&",$queryParam_arr);
        $sign = md5($queryParam.'&key='.$key);
        if($sign != $own_sign){
            return false;
        }
        return true;
    }

    /**
     * 记录日志
     * @param $data
     * @param $return_data
     * @param $time
     */
    private function saveRequestData($data,$return_data,$time=0){
        if(!is_string($data)) $data = var_export($data,true);
        if(!is_string($return_data)) $return_data = var_export($return_data,true);

        if(empty($time)) $time = 0;
        $this->log_data = [
            'post_data' => $data,
            'response' => $return_data,
            'time' => $time
        ];
    }

    /**
     * xml转数组
     * @param $xml
     * @return mixed
     */
    private function xmlToArray($xml){
        libxml_disable_entity_loader(true);
        $arr=json_decode(json_encode(simplexml_load_string($xml,null,LIBXML_NOCDATA)),true);
        return $arr;
    }

    /**
     * 根据支付方式获取乐刷的支付配置
     * @param $pay_method
     * @return string
     * @throws LePassPayException
     */
    protected function getPayWay($pay_method){
        //WXZF  微信
        //ZFBZF 支付宝
        //QQZF  QQ
        //UPSMZF    银联二维码
        switch ($pay_method){
            case '微信支付':
                return "WXZF";
            case '支付宝支付':
                return "ZFBZF";
            case 'QQ支付':
                return "QQZF";
            case '银联支付':
                return "UPSMZF";
            default :
                throw new LePassPayException('错误的支付方式');
        }
    }

    /**
     * 获取随机字符串
     * @param int $len
     * @return string
     */
    protected function getNonceStr($len=32){
        $chars = "abcdefghijklmnopqrstuvwxyz0123456789";
        $str ="";
        for ( $i = 0; $i < $len; $i++ )  {
            $str.= substr($chars, mt_rand(0, strlen($chars)-1), 1);
        }
        return $str;
    }
}